import{ world, system, MinecraftEffectTypes, MinecraftBlockTypes  } from "@minecraft/server";
import "./config/loader";

let config = {
	"rns:stamina": "true",
	"rns:stamina_exhausted": "1",
	"rns:stamina_cooldown": "1",
	"rns:stamina_strength": "320",
	"rns:roll": "true",
	"rns:roll_vertical_strength": "10",
	"rns:roll_horizontal_strength": "10",
	"rns:show_cooldown": "true",
	"rns:slow_damage": "true"
};

function getActor(){
	let actor;
	for(let actor_type of world.getDimension("minecraft:overworld").getEntitiesAtBlockLocation({x:0, y:320, z:0})){
		if(actor_type.typeId == "kc:config_loader"){
			actor = actor_type;
			return actor;
		}
	}
	return actor;
}

function getConfigFromActor(){
	let l = {};
	let actor = getActor();
	if(actor == undefined) return {};
	for(let a of actor.getTags()){
		let b = a.split("=");
		l[b[0]] = b[1];
	}
	return l;
}
system.events.scriptEventReceive.subscribe( s => {
	config = getConfigFromActor();
}, { namespaces: [ "config_loaded" ]});

let playerHurt = [];
let rollCooldown = [];
let rollTime = [];
let staminaDuration = [];
let stamina_is_cooldown = [];
let player_on_ground = [];
let roll_cooldown_actionbar = [
	"textures/ui/roll/cooldown_0",
	"textures/ui/roll/cooldown_1",
	"textures/ui/roll/cooldown_2",
	"textures/ui/roll/cooldown_3",
	"textures/ui/roll/cooldown_4",
	"textures/ui/roll/cooldown_5"
];

let ui_temp = {};

function setUI(playerData, path, id){
	
	if(!ui_temp[playerData.id]){
		ui_temp[playerData.id] = { [id]: path };
	}else{
		if(ui_temp[playerData.id][id] == path) return;
	}
//	console.warn(path)
	playerData.runCommand("scriptevent ui_load:" + id + " " + path + id);
	ui_temp[playerData.id][id] = path;
}

system.runInterval(() => {
	let creative_player = world.getPlayers({gameMode: "creative"});
	let stamina_strength = Number(config["rns:stamina_strength"]);
	let stamina_exhausted = Number(config["rns:stamina_exhausted"])*0.5+0.5;
	let stamina_cooldown = Number(config["rns:stamina_cooldown"])*0.5+0.5;
	for(let playerData of world.getPlayers()){
		let is_creative = false;
		for(let player of creative_player){
			if(player.id == playerData.id){
				is_creative = true;
				break;
			}
		}
		
		let is_riding = false;
		for(let ride of playerData.dimension.getEntities({ location: playerData.location, excludeTypes: ["minecraft:player"], maxDistance: 2})){
			if(ride.hasComponent("minecraft:rideable")){
				is_riding = true;
				break;
			}
		}
		let vel = playerData.getVelocity();
		let is_in_liquid = playerData.dimension.getBlock(playerData.location).isLiquid();
		let player_speed = playerData.getComponent("minecraft:movement");
		let vel_calc = Math.sqrt(Math.pow(vel.x, 2) + Math.pow(vel.z, 2));
		let is_sprint = (vel_calc - player_speed.current*2) > 0.0175;
		if(!player_on_ground[playerData.id] && Math.abs(vel.y) < 0.005){
			player_on_ground[playerData.id] = true;
		}
		if(config["rns:stamina"] == "true"){
			if(!is_creative){
				if(player_on_ground[playerData.id] && vel.y > 0.25 && !playerData.getEffect(MinecraftEffectTypes.jumpBoost) && !is_riding){
					player_on_ground[playerData.id] = false;
					staminaDuration[playerData.id] += 10*stamina_exhausted;
				}
				
				if(staminaDuration[playerData.id] == undefined) staminaDuration[playerData.id] = 0;
				if(is_sprint && !is_riding){
					if(staminaDuration[playerData.id] < stamina_strength && !is_in_liquid && !playerData.getEffect(MinecraftEffectTypes.speed)) staminaDuration[playerData.id] += 1.5*stamina_exhausted;
				}else{
					if(staminaDuration[playerData.id] > -1) staminaDuration[playerData.id] -= ((playerData.isSneaking) ? 1 : 0.5)*stamina_cooldown;
				}
				
				if(stamina_is_cooldown[playerData.id] && vel.y > 0 && !is_in_liquid) playerData.applyKnockback(0, 0, 0, -1);
				if(staminaDuration[playerData.id] >= stamina_strength){
					staminaDuration[playerData.id] = stamina_strength;
					let speed_value = player_speed.value;
					let speed_decreased = speed_value/1.5;
					let runLoop = system.runInterval(() => {
						player_speed.setCurrent(speed_decreased);
						stamina_is_cooldown[playerData.id] = true;
					});
					let run = system.runInterval(() => {
						system.clearRun(runLoop);
						player_speed.setCurrent(speed_value);
						stamina_is_cooldown[playerData.id] = false;
						system.clearRun(run);
					}, 140);
				}
				if(staminaDuration[playerData.id] >= 0){
					let cooldown_delta = (staminaDuration[playerData.id])/stamina_strength;
					cooldown_delta = 100-Math.floor(Math.min(cooldown_delta, 1)*100);
					if(config["rns:show_cooldown"] == "true") setUI(playerData, "textures/ui/stamina/bar_" + cooldown_delta, "stamina_update");
				}
			}
		}
		if(config["rns:roll"] == "false") continue;
		
		let time = system.currentTick;
		if(rollCooldown[playerData.id] != -1 && rollCooldown[playerData.id] != undefined){
			let cooldown_delta = time - rollCooldown[playerData.id];
			cooldown_delta = Math.floor(cooldown_delta/10);
			if(cooldown_delta <= 5){
				if(config["rns:show_cooldown"] == "true") setUI(playerData, roll_cooldown_actionbar[cooldown_delta], "roll_update");
			}else{
				rollCooldown[playerData.id] = -1;
			}
			return;
		}
		
		if(is_in_liquid) return;
		
		if(playerData.isSneaking){
			if(rollTime[playerData.id] == -1 || rollTime[playerData.id] == undefined){
				rollTime[playerData.id] = time;
			}
		}else{
			if(rollTime[playerData.id] != -1 && rollTime[playerData.id] != undefined){
				if(time - rollTime[playerData.id] < 5){
					let rot = playerData.getRotation();
					let rot_temp = Math.floor((Math.atan2(vel.z, vel.x) * 180 / Math.PI - 90));
					if(rot_temp < -180){
						rot_temp = rot_temp+360;
					}
					let delta_rot = Math.abs(rot_temp - Math.floor((rot.y)));
					delta_rot = Math.min(delta_rot, (360-delta_rot));
					let power = 1;
					let strength = (1-Math.min(Math.abs(vel.y*3), 1))*2.3+0.2;
					let roll_anim = (delta_rot > 120) ? "animation.humanoid.roll_down" : "animation.humanoid.roll_up";
					let roll_anim_def = (delta_rot > 120) ? "animation.humanoid.roll_steady2" : "animation.humanoid.roll_steady";
					if(Math.abs(vel.x) + Math.abs(vel.z) > 0.01){
						let vel_up = vel_calc*2;
						playerData.applyKnockback(vel.x, vel.z, strength * ((delta_rot > 120) ? 1.5 : 1)*0.1*Number(config["rns:roll_horizontal_strength"]), ((delta_rot > 120) ? 0.2 : 0.4)*0.1*Number(config["rns:roll_vertical_strength"]));
					}else{
						rot.y = (rot.y + 45)* Math.PI/180;
						let velocity = {
							"x": (Math.cos(rot.y)-Math.sin(rot.y))*power,
							"y": 0,
							"z": (Math.sin(rot.y)+Math.cos(rot.y))*power,
						};
						playerData.applyKnockback(velocity.x, velocity.z, strength/2*0.1*Number(config["rns:roll_horizontal_strength"]), 0.6*0.1*Number(config["rns:roll_vertical_strength"]));
						roll_anim = "animation.humanoid.roll_up";
					}
					rollCooldown[playerData.id] = system.currentTick;
					playerData.playAnimation(roll_anim_def, {
						blendOutTime: 0.3,
						controller: "roll_steady_controller"
					});
					playerData.playAnimation(roll_anim, {
						controller: "roll_controller"
					});
					playerData.playAnimation("animation.humanoid.roll_head", {
						blendOutTime: 1,
						controller: "roll_head_controller"
					});
					if(!playerData.getEffect(MinecraftEffectTypes.jumpBoost) && config["rns:stamina"] == "true") staminaDuration[playerData.id] += 10*stamina_exhausted;
				}
				rollTime[playerData.id] = -1; 
			}
		}
	}
});

world.afterEvents.entityHurt.subscribe( s => {
	if(config["rns:slow_damage"] == "false") return;
	let player = s.hurtEntity;
	if(s.damageSource.cause == "magic" || playerHurt[player.id]) return;
	let player_speed = player.getComponent("minecraft:movement");
	let speed_value = player_speed.value;
	let speed_decreased = speed_value/1.5;
	let runLoop = system.runInterval(() => {
		player_speed.setCurrent(speed_decreased);
		playerHurt[player.id] = true;
	});
	let run = system.runInterval(() => {
		system.clearRun(runLoop);
		player_speed.setCurrent(speed_value);
		playerHurt[player.id] = false;
		system.clearRun(run);
	}, 30);
}, {entityTypes: [ "minecraft:player" ]});
